function ROC_data = roc_curve(class_1, class_2, dispp, dispt)
if(nargin<4), dispt = 1;    end
if(nargin<3), dispp = 1;    end
if(nargin<2), error('Params "class_1" or "class_2" are not indicated.'); end
class_1 = class_1(:);
class_2 = class_2(:);
s_data = unique(sort([class_1; class_2]));
s_data(isnan(s_data)) = [];
d_data = diff(s_data);
if(isempty(d_data)), error('Both class data are the same!'); end
d_data(length(d_data)+1,1) = d_data(length(d_data));
thres(1,1) = s_data(1) - d_data(1);
thres(2:length(s_data)+1,1) = s_data + d_data./2;
curve = zeros(size(thres,1),2);
distance = zeros(size(thres,1),1);
for id_t = 1:1:length(thres)
    TP = length(find(class_2 >= thres(id_t)));
    FP = length(find(class_1 >= thres(id_t)));
    FN = length(find(class_2 < thres(id_t)));
    TN = length(find(class_1 < thres(id_t)));    
    curve(id_t,1) = TP/(TP + FN);
    curve(id_t,2) = TN/(TN + FP);    
    distance(id_t)= sqrt((1-curve(id_t,1))^2+(curve(id_t,2)-1)^2);
end
[~, opt] = min(distance);
TP = length(find(class_2 >= thres(opt)));
FP = length(find(class_1 >= thres(opt)));
FN = length(find(class_2 < thres(opt)));
TN = length(find(class_1 < thres(opt)));
param.Threshold = thres(opt);
param.Sensi = curve(opt,1);
param.Speci = curve(opt,2);
param.AROC  = abs(trapz(1-curve(:,2),curve(:,1)));
param.Accuracy = (TP+TN)/(TP+TN+FP+FN);
param.PPV   = TP/(TP+FP);
param.NPV   = TN/(TN+FN);
param.FNR   = FN/(FN+TP);
param.FPR   = FP/(FP+TN);
param.FDR   = FP/(FP+TP);
param.FOR   = FN/(FN+TN);
param.F1_score = 2*TP/(2*TP+FP+FN);
param.MCC   = (TP*TN-FP*FN)/sqrt((TP+FP)*(TP+FN)*(TN+FP)*(TN+FN));
param.BM    = param.Sensi+param.Speci-1;
param.MK    = param.PPV+param.NPV-1;
param.TP = TP;
param.FP = FP;
param.FN = FN;
param.TN = TN;
if(dispp == 1)
    fill_color = [0.90 0.89 0.65];    
    fill([1-curve(:,2); 1], [curve(:,1); 0], fill_color,'FaceAlpha',0.5);
    hold on; plot(1-curve(:,2), curve(:,1), 'r-', 'LineWidth', 2);
    hold on; plot(1-curve(opt,2), curve(opt,1), 'bs', 'MarkerSize', 6);
    hold on; plot(1-curve(opt,2), curve(opt,1), 'bo', 'MarkerSize', 8);
    hold off; axis square; grid on; xlabel('True Positive Rate'); ylabel('False Positive Rate)');
    title('ROC Curve')
    grid minor;
end

if param.AROC < 0.5
    warning('Since AROC is less than 0.5, you should swap the classes: roc_curve(class_2,class_1).');
end
ROC_data.param = param;
ROC_data.curve = curve;
end